package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.dto.MusicCardDto;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface MusicGroupDao extends BaseDAO<String, MusicGroup> {

    /**
     * 获取学员所在乐团详情列表
     * @param userId
     * @return
     */
    List<MusicCardDto> queryUserMusicGroups(Integer userId);

    /**
     * 获取个人中心乐团数据
     * @param id
     * @return
     */
    MusicCardDto queryPersonalMusicGroups(Integer id);

    /**
     * 获取用户有效的乐团数
     * @param userId
     * @return
     */
    Integer countUserMusicGroups(Integer userId);

    /**
     * 根据课程类型获取剩余课时
     * @param userId
     * @param type
     * @return
     */
    Integer countCourseByType(@Param("userId") Integer userId, @Param("type") ClassGroupTypeEnum type);

    /**
     * 根绝学员编号获取相关老师列表
     * @param studentId
     * @return
     */
    List<BasicUserDto> findTeachersByStuId(Integer studentId);

    /**
     * 获取用户所在乐团
     * @param userId
     * @return
     */
    List<ImGroupModel> queryUserGroups(@Param("userId") Integer userId,@Param("search") String search);

    /**
     * 获取乐团总人数
     * @param musicGroupId
     * @return
     */
    Integer countMusicGroupUserNum(String musicGroupId);

    /**
     * 获取乐团所有学员基本信息
     * @param musicGroupId
     * @return
     */
    List<ImUserModel> findGroupUsers(String musicGroupId);
}