package com.ym.mec.biz.service;

import java.util.List;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.page.StudentPayLogQueryInfo;
import com.ym.mec.biz.dal.page.VipClassQueryInfo;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.BaseService;

public interface ClassGroupService extends BaseService<Integer, ClassGroup> {

    /**
     * 查询老师乐团课课程
     *
     * @param teacherId
     * @return
     */
    List<ClassGroup> findClassGroup4Teacher(Integer teacherId);

    /**
     * 合奏班相关班级获取
     *
     * @param mixClassGroupId
     * @return
     */
    List<ClassGroup> findClassGroup(Integer musicGroupId, Integer mixClassGroupId);

    /**
     * 根据乐团编号查询单技班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllNormalClassGroupByMusicGroupId(int musicGroupId);


    /**
     * 根据乐团编号查询合奏班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllMixClassGroupByMusicGroupId(int musicGroupId);

    /**
     * 添加单技班
     *
     * @param classGroup
     * @return
     */
    ClassGroup addClassGroup(ClassGroup classGroup) throws Exception;

    /**
     * 添加合奏班
     *
     * @param classGroupIds
     * @return
     * @throws Exception
     */
    ClassGroup addMixClassGroup(int musicGroupId, String name, String classGroupIds) throws Exception;

    /**
     * 跟班级ids查询班级信息
     *
     * @param ids
     * @return
     */
    List<ClassGroup> findClassGroupByIds(String ids);


    /**
     * 提高班批量添加
     *
     * @param highClassGroupList
     * @return
     * @throws Exception
     */
    List<HighClassGroupDto> addHighClassGroup(List<HighClassGroupDto> highClassGroupList) throws Exception;

    /**
     * 获取老师乐团课程信息列表
     *
     * @param teacherId
     * @return
     */
    List<TeacherMusicClassInfoDto> getTeacherMusicClass(Integer teacherId);

    /**
     * 获取老师VIP课程信息列表
     *
     * @param queryInfo
     * @return
     */
    PageInfo<TeacherVipClassInfoDto> getTeacherVipClass(VipClassQueryInfo queryInfo);


    /**
     * 获取乐团班级老师课酬
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroupTeachersDto> getClassGroupAndTeachers(Integer musicGroupId);

    /**
     * 获取乐团班级老师课酬
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroupTeachersDto> getClassGroupAndTeacherSalary(Integer musicGroupId);


    /**
     * 获取学生所购买课程列表
     *
     * @param userId
     * @param queryInfo
     * @return
     */
    PageInfo<CourseListDto> queryCoursePage(QueryInfo queryInfo,Integer userId);

    /**
     * 获取学员所在群组信息
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryUserGroups(Integer userId,String search);

    /**
     * 获取群组下用户基本信息
     * @param groupId
     * @return
     */
    List<ImUserModel> findGroupUsers(String groupId);

    /**
     * 分页查询学生购买记录
     * @param studentPayLogQueryInfo
     * @return
     */
    PageInfo<StudentPayInfoDto> queryStudentPayLog(StudentPayLogQueryInfo studentPayLogQueryInfo);
}