package com.ym.mec.biz.service.impl;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.StudentInfo;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.SysUserContracts;
import com.ym.mec.biz.dal.entity.SysUserContracts.ContractType;
import com.ym.mec.biz.dal.entity.SysUserTsign;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.service.ContractService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.biz.service.SysUserContractsService;
import com.ym.mec.biz.service.SysUserTsignService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.eseal.ESealPlugin;
import com.ym.mec.thirdparty.storage.StoragePlugin;
import com.ym.mec.util.freemarker.FreemarkerTemplateEngine;
import com.ym.mec.util.pdf.PDFUtil;

@Service
public class ContractServiceImpl implements ContractService {

	@Autowired
	private SysUserTsignService sysUserTsignService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private SysUserContractsService sysUserContractsService;
	
	@Autowired
	private StudentRegistrationService studentRegistrationService;
	
	@Autowired
	private StudentPaymentOrderService studentPaymentOrderService;
	
	@Autowired
	private SubjectService subjectService;

	@Autowired
	private ESealPlugin eSealPlugin;

	@Autowired
	private StoragePlugin storagePlugin;

	@Value("${contract.baseDir:/var/pdf}")
	private String contractBaseDir;

	private DateFormat dateFormatOss = new SimpleDateFormat("yyyy/MM/dd");

	@Override
	public boolean register(Integer userId) {

		SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

		if (sysUserTsign == null) {

			SysUser user = sysUserFeignService.queryUserById(userId);
			if (user == null) {
				return false;
			}
			String accountId = eSealPlugin.createUserAccount(user.getRealName(), user.getIdCardNo(), user.getPhone());

			if (StringUtils.isBlank(accountId)) {
				throw new BizException("创建电子存证账户失败");
			}

			String sealData = eSealPlugin.createSeal(accountId);

			if (StringUtils.isBlank(sealData)) {
				throw new BizException("创建电子存证印章失败");
			}

			sysUserTsign = new SysUserTsign(userId, accountId, sealData);

			sysUserTsignService.insert(sysUserTsign);
		}

		return true;
	}

	@Override
	public boolean transferRegisterContract(Integer userId) {

		SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

		if (sysUserTsign == null) {
			return false;
		}

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/register/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();

		templateEngine.render(params, "register.ftl", new File(srcPdfPath));

		// 生成借款协议PDF
		try {
			PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(new File(srcPdfPath)), srcPdfPath,
					"config/fonts/simsun.ttc");
		} catch (IOException e) {
			throw new BizException("生成pdf协议失败", e);
		}

		eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);

		Date date = new Date();
		// 上传到oss
		String dateStrOss = dateFormatOss.format(date);
		String pdfFilePath = storagePlugin.uploadFile(dateStrOss, new File(srcPdfPath));

		SysUserContracts sysUserContracts = new SysUserContracts();
		sysUserContracts.setCreateTime(date);
		sysUserContracts.setType(ContractType.REGISTER);
		sysUserContracts.setUrl(pdfFilePath);
		sysUserContracts.setUserId(userId);

		sysUserContractsService.insert(sysUserContracts);

		FileUtils.deleteQuietly(new File(srcPdfPath));

		return true;
	}

	@Override
	public String queryRegisterContract(Integer userId) {

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/register/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();

		templateEngine.render(params, "register.ftl", new File(srcPdfPath));

		String result = "";
		try {
			result = FileUtils.readFileToString(new File(srcPdfPath));
		} catch (IOException e) {
			throw new BizException("读取注册协议出错", e);
		}
		FileUtils.deleteQuietly(new File(srcPdfPath));

		return result;
	}

	@Override
	public boolean transferMusicGroupCoursesContract(Integer userId, Integer musicGroupId) {
		SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

		if (sysUserTsign == null) {
			return false;
		}

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/courses/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();
		
		//查询参数信息
		StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
		params.put("studentInfo", studentInfo);

		templateEngine.render(params, "courses.ftl", new File(srcPdfPath));

		// 生成借款协议PDF
		try {
			PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(new File(srcPdfPath)), srcPdfPath,
					"config/fonts/simsun.ttc");
		} catch (IOException e) {
			throw new BizException("生成pdf协议失败", e);
		}

		eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);

		Date date = new Date();
		// 上传到oss
		String dateStrOss = dateFormatOss.format(date);
		String pdfFilePath = storagePlugin.uploadFile(dateStrOss, new File(srcPdfPath));

		SysUserContracts sysUserContracts = new SysUserContracts();
		sysUserContracts.setCreateTime(date);
		sysUserContracts.setType(ContractType.REGISTER);
		sysUserContracts.setUrl(pdfFilePath);
		sysUserContracts.setUserId(userId);

		sysUserContractsService.insert(sysUserContracts);

		FileUtils.deleteQuietly(new File(srcPdfPath));

		return true;
	}

	@Override
	public String queryMusicGroupCoursesContract(Integer userId, Integer musicGroupId) {

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/courses/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();
		
		StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
		params.put("studentInfo", studentInfo);

		templateEngine.render(params, "courses.ftl", new File(srcPdfPath));

		String result = "";
		try {
			result = FileUtils.readFileToString(new File(srcPdfPath));
		} catch (IOException e) {
			throw new BizException("读取课程协议出错", e);
		}
		FileUtils.deleteQuietly(new File(srcPdfPath));

		return result;
	}

	@Override
	public boolean transferGoodsContract(Integer userId, Integer musicGroupId) {
		SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

		if (sysUserTsign == null) {
			return false;
		}

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/goods/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();
		
		//查询参数信息
		StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
		
		Subject subject = subjectService.get(studentInfo.getSubject().getId());
		studentInfo.setSubject(subject);
		
		params.put("studentInfo", studentInfo);
		
		List<Goods> goodsList = studentPaymentOrderService.queryApplyGoodsList(musicGroupId, OrderDetailTypeEnum.MUSICAL);
		params.put("goodsList", goodsList);

		templateEngine.render(params, "goods.ftl", new File(srcPdfPath));

		// 生成借款协议PDF
		try {
			PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(new File(srcPdfPath)), srcPdfPath,
					"config/fonts/simsun.ttc");
		} catch (IOException e) {
			throw new BizException("生成pdf协议失败", e);
		}

		eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);

		Date date = new Date();
		// 上传到oss
		String dateStrOss = dateFormatOss.format(date);
		String pdfFilePath = storagePlugin.uploadFile(dateStrOss, new File(srcPdfPath));

		SysUserContracts sysUserContracts = new SysUserContracts();
		sysUserContracts.setCreateTime(date);
		sysUserContracts.setType(ContractType.REGISTER);
		sysUserContracts.setUrl(pdfFilePath);
		sysUserContracts.setUserId(userId);

		sysUserContractsService.insert(sysUserContracts);

		FileUtils.deleteQuietly(new File(srcPdfPath));

		return true;
	}

	@Override
	public String queryGoodsContract(Integer userId, Integer musicGroupId) {

		// 合成freemarker
		String srcPdfPath = contractBaseDir + "/goods/" + userId + ".pdf";
		File debtFile = new File(contractBaseDir);
		if (!debtFile.exists()) {
			debtFile.mkdirs();
		}

		FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
		templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

		Map<String, Object> params = new HashMap<String, Object>();

		StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
		params.put("studentInfo", studentInfo);
		
		List<Goods> goodsList = studentPaymentOrderService.queryApplyGoodsList(musicGroupId, OrderDetailTypeEnum.MUSICAL);
		params.put("goodsList", goodsList);

		templateEngine.render(params, "goods.ftl", new File(srcPdfPath));

		String result = "";
		try {
			result = FileUtils.readFileToString(new File(srcPdfPath));
		} catch (IOException e) {
			throw new BizException("读取商品协议出错", e);
		}
		FileUtils.deleteQuietly(new File(srcPdfPath));

		return result;
	}

}
