package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.dto.MusicCardDto;
import com.ym.mec.biz.dal.dto.SubFeeSettingDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupMember;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Service
public class MusicGroupServiceImpl extends BaseServiceImpl<String, MusicGroup>  implements MusicGroupService {
	
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
	@Autowired
	private MusicGroupPaymentEntitiesDao musicGroupPaymentEntitiesDao;
	@Autowired
	private MusicGroupSubjectGoodsGroupDao musicGroupSubjectGoodsGroupDao;
	@Autowired
	private MusicGroupSubjectPlanDao musicGroupSubjectPlanDao;
	@Autowired
	private ImFeignService imFeignService;
	@Autowired
	private MusicGroupBuildLogDao musicGroupBuildLogDao;

	@Override
	public BaseDAO<String, MusicGroup> getDAO() {
		return musicGroupDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void createGroup(SubFeeSettingDto subFeeSettingDto,Integer userId) {
		MusicGroup musicGroup = subFeeSettingDto.getMusicGroup();
		List<MusicGroupPaymentEntities> musicGroupPaymentEntities = subFeeSettingDto.getMusicGroupPaymentEntities();
		List<MusicGroupSubjectGoodsGroup> musicGroupSubjectGoodsGroups = subFeeSettingDto.getMusicGroupSubjectGoodsGroups();
		List<MusicGroupSubjectPlan> musicGroupSubjectPlans = subFeeSettingDto.getMusicGroupSubjectPlans();
		if(musicGroup == null || musicGroupPaymentEntities == null || musicGroupSubjectGoodsGroups == null || musicGroupSubjectPlans == null){
			String musicGroupId = UUID.randomUUID().toString();
			musicGroup.setId(musicGroupId);
			//保存乐团基本信息
			musicGroupDao.insert(musicGroup);
			//保存乐团付费主体列表
			musicGroupPaymentEntities.forEach(e ->{
				e.setMusicGroupId(musicGroupId);
				musicGroupPaymentEntitiesDao.insert(e);
			});
			//保存学员付费周期
			List<Integer> months = subFeeSettingDto.getMonths();
			if(months != null && months.size() > 0){
				MusicGroupPaymentCalender mg;
				for (Integer month:months) {
					mg = new MusicGroupPaymentCalender();
					mg.setMusicGroupId(musicGroupId);
					mg.setPaymentMonth(month);
					musicGroupPaymentCalenderDao.insert(mg);
				}
			}
			//保存乐团声部规划
			if (musicGroupSubjectPlans == null) {
				musicGroupSubjectPlans = new ArrayList<MusicGroupSubjectPlan>();
			}
			musicGroupSubjectPlans.forEach(e ->{
				e.setMusicGroupId(musicGroupId);
				musicGroupSubjectPlanDao.insert(e);
			});
			//保存乐团声部商品规划
			if (musicGroupSubjectGoodsGroups == null) {
				musicGroupSubjectGoodsGroups = new ArrayList<MusicGroupSubjectGoodsGroup>();
			}
			musicGroupSubjectGoodsGroups.forEach(e->{
				e.setMusicGroupId(musicGroupId);
				musicGroupSubjectGoodsGroupDao.insert(e);
			});
			//新增聊天群
			ImGroupModel imGroupModel = new ImGroupModel(musicGroupId, musicGroup.getName());
			//教务老师和运营主管加入群组
			ImGroupMember[] imGroupMembers = {new ImGroupMember(musicGroup.getTeamTeacherId().toString()),
					new ImGroupMember(musicGroup.getEducationalTeacherId().toString())};
			//创建群组
			imFeignService.groupCreate(new ImGroupModel(musicGroupId,imGroupMembers,musicGroup.getName()));
			//记录创建日志
			musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroupId,"提交建团申请",userId,""));
		}
	}

	@Override
	public List<MusicCardDto> queryUserMusicGroups(Integer userId) {
		return musicGroupDao.queryUserMusicGroups(userId);
	}

	@Override
	public MusicCardDto queryPersonalMusicGroups(Integer id) {
		MusicCardDto musicCardDto = musicGroupDao.queryPersonalMusicGroups(id);
		musicCardDto.setMusicGroupNum(musicGroupDao.countUserMusicGroups(id));
		musicCardDto.setVipClassNum(musicGroupDao.countCourseByType(id,ClassGroupTypeEnum.VIP));
		return musicGroupDao.queryPersonalMusicGroups(id);
	}

	@Override
	public List<BasicUserDto> findTeachersByStuId(Integer studentId) {
		return musicGroupDao.findTeachersByStuId(studentId);
	}
}