package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.ym.mec.biz.dal.dto.MusicGroupRegRespDto;
import com.ym.mec.biz.dal.dto.MusicGroupSubjectGoodsAndInfoDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.MusicGroupSubjectPlanDao;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;

import java.util.List;
import java.util.Map;

@Service
public class MusicGroupSubjectPlanServiceImpl extends BaseServiceImpl<Integer, MusicGroupSubjectPlan> implements MusicGroupSubjectPlanService {

    @Autowired
    private MusicGroupSubjectPlanDao musicGroupSubjectPlanDao;

    @Autowired
    private StudentRegistrationService studentRegistrationService;

    @Autowired
    private GoodsService goodsService;

    @Autowired
    private MusicGroupSubjectGoodsGroupService musicGroupSubjectGoodsGroupService;

    @Autowired
    private MusicGroupService musicGroupService;

    @Autowired
    private SubjectService subjectService;

    @Override
    public BaseDAO<Integer, MusicGroupSubjectPlan> getDAO() {
        return musicGroupSubjectPlanDao;
    }

    @Override
    public List<MusicGroupSubjectPlan> getMusicSubjectClassPlan(int musicGroupId) {
        List<MusicGroupSubjectPlan> musicSubjectClassPlans = musicGroupSubjectPlanDao.getMusicSubjectClassPlan(musicGroupId);
        for (MusicGroupSubjectPlan musicSubjectClassPlan : musicSubjectClassPlans) {
            Integer noClassStuCount = studentRegistrationService.getNoClassStuCountBySubjectId(musicGroupId, musicSubjectClassPlan.getSubjectId());
            musicSubjectClassPlan.setNoClassStudentNum(noClassStuCount);
        }
        return musicSubjectClassPlans;
    }

    @Override
    public MusicGroupSubjectPlan getMusicOneSubjectClassPlan(int musicGroupId, int subjectId) {
        return musicGroupSubjectPlanDao.getMusicOneSubjectClassPlan(musicGroupId, subjectId);
    }

    @Override
    public MusicGroupSubjectGoodsAndInfoDto getSubjectGoodsAndInfo(Integer musicGroupId, Integer subjectId) {
        MusicGroup musicGroup = musicGroupService.get(musicGroupId.toString());

        //课程形态
        Map<String, Object> courseForm = JSON.parseObject(musicGroup.getCourseForm(), Map.class);

        //乐团计划及收费信息
        MusicGroupSubjectPlan musicOneSubjectClassPlan = this.getMusicOneSubjectClassPlan(musicGroupId, subjectId);

        //乐团乐器及辅件信息
        List<MusicGroupSubjectGoodsGroup> goodsGroups = musicGroupSubjectGoodsGroupService.findGoodsGroup(musicGroupId, subjectId);
        goodsGroups.forEach(goodsGroup -> {
            if (goodsGroup.getType().equals(GoodsType.INSTRUMENT)) {
                goodsGroup.getGoodsList().forEach(goods -> {
                    goods.setGoodsList(goodsService.findGoodsByIds(goods.getComplementGoodsIdList()));
                });
            }
        });
        MusicGroupSubjectGoodsAndInfoDto musicGroupSubjectGoodsAndInfo = new MusicGroupSubjectGoodsAndInfoDto();
        musicGroupSubjectGoodsAndInfo.setMusicGroupSubjectPlan(musicOneSubjectClassPlan);
        musicGroupSubjectGoodsAndInfo.setCourseScheduleInfo(courseForm);
        musicGroupSubjectGoodsAndInfo.setMusicGroupSubjectGoodsGroupList(goodsGroups);
        return musicGroupSubjectGoodsAndInfo;
    }

    @Override
    public MusicGroupRegRespDto getMusicGroupRegInfo(Integer musicGroupId) {
        MusicGroupRegRespDto musicGroupRegResp = new MusicGroupRegRespDto();

        //乐团信息
        MusicGroup musicGroup = musicGroupService.get(musicGroupId.toString());
        musicGroupRegResp.setMusicGroupName(musicGroup.getName());
        //声部信息
        List<Subject> subjectList = subjectService.findSubByMusicGroupId(musicGroupId);
        musicGroupRegResp.setSubjectList(subjectList);

        return musicGroupRegResp;
    }
}