package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.StudentVipGroupQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupAttendanceQueryInfo;
import com.ym.mec.biz.service.CourseScheduleService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class VipGroupServiceImpl extends BaseServiceImpl<Long, VipGroup> implements VipGroupService {
	
	@Autowired
	private VipGroupDao vipGroupDao;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private VipGroupClassGroupMapperDao vipGroupClassGroupMapperDao;
	@Autowired
	private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
	@Autowired
	private CourseScheduleService courseScheduleService;
	@Autowired
	private VipGroupDefaultClassesUnitPriceDao vipGroupDefaultClassesUnitPriceDao;
	@Autowired
	private VipGroupActivityDao vipGroupActivityDao;
	@Autowired
	private VipGroupCategoryDao vipGroupCategoryDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private TeacherDefaultVipGroupSalaryDao teacherDefaultVipGroupSalaryDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private StudentVipGroupPaymentDao studentVipGroupPaymentDao;
    @Autowired
	private StudentApplyRefundsDao studentApplyRefundsDao;
    @Autowired
	private SysUserCashAccountDao sysUserCashAccountDao;
    @Autowired
	private SysUserCashAccountDetailDao sysUserCashAccountDetailDao;

	@Override
	public BaseDAO<Long, VipGroup> getDAO() {
		return vipGroupDao;
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void createVipGroup(VipGroupApplyDto vipGroup) {

	    if(vipGroup.getCourseSchedules().size()!=vipGroup.getVipGroupApplyBaseInfo().getAllCourseNum()){
	        throw new BizException("课时总数不匹配!");
        }

        Map<TeachModeEnum, List<CourseSchedule>> courseScheduleGroup = vipGroup.getCourseSchedules().stream().collect(Collectors.groupingBy(CourseSchedule::getTeachMode));
	    if(courseScheduleGroup.get(TeachModeEnum.OFFLINE).size()<vipGroup.getVipGroupApplyBaseInfo().getOfflineClassesNum()){
	        throw new BizException("线下课课时数量安排有误!");
        }
        if(courseScheduleGroup.get(TeachModeEnum.ONLINE).size()<vipGroup.getVipGroupApplyBaseInfo().getOnlineClassesNum()){
            throw new BizException("线上课课时数量安排有误!");
        }

        VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupApplyBaseInfo().getVipGroupActivityId().intValue());
		VipGroupApplyBaseInfoDto vipGroupApplyBaseInfoDto=vipGroup.getVipGroupApplyBaseInfo();
		vipGroupDao.insert(vipGroupApplyBaseInfoDto);
		VipGroupCategory vipGroupCategory = vipGroupCategoryDao.get(vipGroup.getVipGroupApplyBaseInfo().getVipGroupCategoryId().intValue());
		ClassGroup classGroup=new ClassGroup();
		classGroup.setExpectStudentNum(vipGroupCategory.getStudentNum());
		classGroup.setName(vipGroupApplyBaseInfoDto.getName());
		classGroup.setExpectStudentNum(vipGroupApplyBaseInfoDto.getExpectStudentNum());
		classGroup.setType(ClassGroupTypeEnum.VIP);
		classGroup.setDelFlag(YesOrNoEnum.NO);
		classGroupDao.insert(classGroup);
		VipGroupClassGroupMapper vipGroupClassGroupMapper=new VipGroupClassGroupMapper();
		vipGroupClassGroupMapper.setVipGroupId(vipGroupApplyBaseInfoDto.getId());
		vipGroupClassGroupMapper.setClassGroupId(classGroup.getId());
		vipGroupClassGroupMapperDao.insert(vipGroupClassGroupMapper);
		ClassGroupTeacherMapper classGroupTeacherMapper=new ClassGroupTeacherMapper();
		classGroupTeacherMapper.setTeacherRole(TeachTypeEnum.TEACHING);
		classGroupTeacherMapper.setUserId(Integer.parseInt(vipGroupApplyBaseInfoDto.getTeacherId().toString()));
		classGroupTeacherMapper.setSalary(vipGroupApplyBaseInfoDto.getSalary());
		classGroupTeacherMapperDao.insert(classGroupTeacherMapper);
		vipGroup.getCourseSchedules().forEach(courseSchedule -> {
		    if(courseSchedule.getEndClassTime().after(vipGroupActivity.getCoursesEndTime())
                &&courseSchedule.getStartClassTime().before(vipGroupActivity.getCoursesStartTime())){
		        throw new BizException("课时安排时间超出范围!");
            }
			courseSchedule.setClassGroupId(classGroup.getId());
		});
		courseScheduleService.batchAddCourseSchedule(vipGroup.getCourseSchedules());
	}

	@Override
	public PageInfo findStudentVipGroupList(StudentVipGroupQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countVipGorups(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findVipGroups(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public VipGroupManageDetailDto getVipGroupDetail(Long vipGroupId) {
		VipGroupManageDetailDto vipGroupBaseInfo = vipGroupDao.getVipGroupBaseInfo(vipGroupId);
		return vipGroupBaseInfo;
	}

	@Override
	public PageInfo findVipGroupAttendances(VipGroupAttendanceQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countVipGroupAttendance(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findVipGroupAttendances(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public StudentVipGroupDetailDto getVipGroupShowDetail(Long vipGroupId) {
		return vipGroupDao.getVipGroupDetail(vipGroupId);
	}

	@Override
	public VipGroupCostCountDto vipGroupCostCount(VipGroupCostCountParamsDto vipGroupCostCountParamsDto) {
        SysUser user = sysUserFeignService.queryUserInfo();
        TeacherDefaultVipGroupSalary teacherDefaultVipGroupSalary = teacherDefaultVipGroupSalaryDao.findByTeacherAndCategory(user.getId().longValue(), vipGroupCostCountParamsDto.getVipGroupCategoryId());
        VipGroupDefaultClassesUnitPrice vipGroupDefaultClassesUnitPrice = vipGroupDefaultClassesUnitPriceDao.get(vipGroupCostCountParamsDto.getVipGroupDefaultClassesUnitPriceId().intValue());
		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroupCostCountParamsDto.getVipGroupActivityId().intValue());
        VipGroupSalarySettlementDto vipGroupSalarySettlementDto = JSON.parseObject(vipGroupActivity.getSalarySettlementJson(), VipGroupSalarySettlementDto.class);
        BigDecimal offlineClassNum=new BigDecimal(vipGroupCostCountParamsDto.getOfflineClassesNum());
        BigDecimal onlineClassNum=new BigDecimal(vipGroupCostCountParamsDto.getOnlineClassesNum());


        //教师课酬
        BigDecimal teacherOnlineSalary,teacherOfflineSalary,teacherSalary;
        switch (vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSalarySettlementType()){
            case TEACHER_DEFAULT:
                teacherOnlineSalary=teacherDefaultVipGroupSalary.getOnlineClassesSalary().multiply(onlineClassNum);
                break;
            case RATIO_DISCOUNT:
                teacherOnlineSalary=vipGroupDefaultClassesUnitPrice.getOnlineClassesUnitPrice().multiply(vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue()).multiply(onlineClassNum);
                break;
            case FIXED_SALARY:
                teacherOnlineSalary=vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue().multiply(onlineClassNum);
                break;
             default:
                 throw new BizException("未指定课酬结算标准!");
        }
        switch (vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSalarySettlementType()){
            case TEACHER_DEFAULT:
                teacherOfflineSalary=teacherDefaultVipGroupSalary.getOfflineClassesSalary().multiply(offlineClassNum);
                break;
            case RATIO_DISCOUNT:
                teacherOfflineSalary=vipGroupDefaultClassesUnitPrice.getOfflineClassesUnitPrice().multiply(vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue()).multiply(offlineClassNum);
                break;
            case FIXED_SALARY:
                teacherOfflineSalary=vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue().multiply(offlineClassNum);
                break;
            default:
                throw new BizException("未指定课酬结算标准!");
        }
        teacherSalary=teacherOnlineSalary.add(teacherOfflineSalary);
        return new VipGroupCostCountDto(null,teacherSalary);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void buyVipGroup(VipGroupBuyParamsDto vipGroupBuyParams) {
	    SysUser user = sysUserFeignService.queryUserInfo();
        VipGroup vipGroup = vipGroupDao.get(vipGroupBuyParams.getVipGroupId());

        if(vipGroup.getStatus()!=VipGroupStatusEnum.APPLYING){
            throw new BizException("该课程已结束报名!");
        }

        VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupActivityId().intValue());
        StudentPaymentOrder studentPaymentOrder=new StudentPaymentOrder();
        studentPaymentOrder.setUserId(user.getId());
        String orderNo=StringUtils.join(new String[]{user.getId().toString(),String.valueOf(System.currentTimeMillis())});
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
        studentPaymentOrder.setExpectAmount(countVipGroupPredictFee(vipGroup,vipGroup.getOnlineClassesUnitPrice(),vipGroup.getOfflineClassesUnitPrice()));
        if(vipGroupActivity.getType()==VipGroupActivityTypeEnum.DISCOUNT){
            studentPaymentOrder.setActualAmount(studentPaymentOrder.getExpectAmount().multiply(new BigDecimal(vipGroupActivity.getAttribute1())));
        }else{
            studentPaymentOrder.setActualAmount(studentPaymentOrder.getExpectAmount());
        }
        VipGroupClassGroupMapper vipGroupClassGroupMapper = vipGroupClassGroupMapperDao.findByVipGroupId(vipGroup.getId());
        studentPaymentOrder.setMusicGroupId(vipGroup.getId().toString());
        studentPaymentOrder.setClassGroupId(vipGroupClassGroupMapper.getClassGroupId());
        studentPaymentOrderDao.insert(studentPaymentOrder);

        StudentVipGroupPayment studentVipGroupPayment=new StudentVipGroupPayment();
        studentVipGroupPayment.setUserId(user.getId());
		studentVipGroupPayment.setVipGroupId(vipGroup.getId());
		studentVipGroupPayment.setClassGroupId(vipGroupClassGroupMapper.getClassGroupId());
        studentVipGroupPayment.setStudentPaymentOrderId(studentPaymentOrder.getId());
        studentVipGroupPaymentDao.insert(studentVipGroupPayment);

		ClassGroup classGroup = classGroupDao.get(vipGroupClassGroupMapper.getClassGroupId());
		int classStudents = classGroupDao.countClassStudent(vipGroupClassGroupMapper.getClassGroupId().longValue());
		if(classGroup.getExpectStudentNum()==classStudents){
			vipGroup.setStatus(VipGroupStatusEnum.FINISH);
			vipGroupDao.update(vipGroup);
		}
		classGroup.setStudentNum(classGroup.getStudentNum()+1);
		classGroupDao.update(classGroup);

		//生成回调地址
    }

    /**
     * @Author: Joburgess
     * @Date: 2019/10/2
     * 计算VIP课程预计收费
     */
    private BigDecimal countVipGroupPredictFee(VipGroup vipGroup,BigDecimal onlineClassesUnitPrice,BigDecimal offlineClassesUnitPrice){
        BigDecimal offlineClassNum=new BigDecimal(vipGroup.getOfflineClassesNum());
        BigDecimal onlineClassNum=new BigDecimal(vipGroup.getOnlineClassesNum());
        BigDecimal onlineVipGroupCharge = onlineClassesUnitPrice.multiply(onlineClassNum);
        BigDecimal offlineVipGroupCharge = offlineClassesUnitPrice.multiply(offlineClassNum);
        return onlineVipGroupCharge.add(offlineVipGroupCharge);
    }

	@Override
	public void orderCallback(String callbackParams) {
		/*
		根据回调信息调整订单状态及vip课程状态等相关信息
		 */
	}

	@Override
	public void applyRefund(Long vipGroupId,Long studentId) {
    	if(null==studentId){
    		studentId=sysUserFeignService.queryUserInfo().getId().longValue();
		}
		StudentApplyRefunds studentApplyRefunds=new StudentApplyRefunds();
		StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.findByStudentVipGroup(vipGroupId,studentId);
		VipGroup vipGroup=vipGroupDao.get(vipGroupId);
		//剩余课时数
		Map surplusClassTimes=MapUtil.convertMybatisMap(vipGroupDao.countSurplusClassTimes(vipGroupId));
		studentApplyRefunds.setExpectAmount(countVipGroupPredictFee(vipGroup,
				new BigDecimal(surplusClassTimes.get(TeachModeEnum.ONLINE.getMsg()).toString()),
				new BigDecimal(surplusClassTimes.get(TeachModeEnum.OFFLINE.getMsg()).toString())));
		studentApplyRefunds.setStatus(AuditStatusEnum.ING);
		String orderNo=StringUtils.join(new String[]{studentId.toString(),String.valueOf(System.currentTimeMillis())});
		studentApplyRefunds.setOrderNo(orderNo);
		studentApplyRefunds.setOrigPaymentOrderId(studentPaymentOrder.getId());
		studentApplyRefunds.setUserId(studentId.intValue());
		studentApplyRefundsDao.insert(studentApplyRefunds);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void applyRefundAudit(StudentApplyRefunds studentApplyRefunds) {
		StudentApplyRefunds oldStudentApplyRefunds=studentApplyRefundsDao.get(studentApplyRefunds.getId());
		oldStudentApplyRefunds.setStatus(studentApplyRefunds.getStatus());
		if(null==studentApplyRefunds.getActualAmount()){
			oldStudentApplyRefunds.setActualAmount(oldStudentApplyRefunds.getExpectAmount());
		}else{
			oldStudentApplyRefunds.setActualAmount(studentApplyRefunds.getActualAmount());
		}
		oldStudentApplyRefunds.setRemark(studentApplyRefunds.getRemark());
		studentApplyRefundsDao.update(oldStudentApplyRefunds);
		switch (studentApplyRefunds.getStatus()){
			case PASS:
				sysUserCashAccountDao.incrAccount(oldStudentApplyRefunds.getUserId(),oldStudentApplyRefunds.getActualAmount());
				SysUserCashAccount sysUserCashAccount = sysUserCashAccountDao.get(studentApplyRefunds.getId().intValue());
				SysUserCashAccountDetail sysUserCashAccountDetail=new SysUserCashAccountDetail();
				sysUserCashAccountDetail.setUserId(studentApplyRefunds.getUserId());
				sysUserCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.REFUNDS);
				sysUserCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
				sysUserCashAccountDetail.setAmount(oldStudentApplyRefunds.getActualAmount());
				sysUserCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
				sysUserCashAccountDetail.setAttribute(oldStudentApplyRefunds.getId().toString());
				sysUserCashAccountDetailDao.insert(sysUserCashAccountDetail);
				break;
			case REJECT:

				break;
			default:
				throw new BizException("审核状态错误!");
		}
	}
}