package com.ym.mec.education;

import com.spring4all.swagger.EnableSwagger2Doc;
import com.ym.mec.common.redis.service.RedisCache;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.client.RestTemplate;

@MapperScan({"com.ym.mec.education.mapper", "com.ym.mec.biz.dal.dao"})
@SpringBootApplication
@EnableDiscoveryClient
@ComponentScan(basePackages = "com.ym.mec")
@Configuration
@EnableSwagger2Doc
public class EducationApplication {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    public static void main(String[] args) {
        SpringApplication.run(EducationApplication.class, args);
    }

    @Bean
    @LoadBalanced
    public RestTemplate restTemplate() {
        return new RestTemplate();
    }

    @Bean
    @ConditionalOnBean(RedisTemplate.class)
    public RedisCache<String, Object> redisCache() {
        return new RedisCache<String, Object>(redisTemplate);
    }
}
