package com.ym.mec.education.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ym.mec.education.base.BaseResponse;
import com.ym.mec.education.base.PageResponse;
import com.ym.mec.education.entity.*;
import com.ym.mec.education.enums.StudentAttendanceStatusEnum;
import com.ym.mec.education.enums.TeachTypeEnum;
import com.ym.mec.education.mapper.CourseScheduleMapper;
import com.ym.mec.education.req.ClassGroupReq;
import com.ym.mec.education.req.CourseScheduleReq;
import com.ym.mec.education.resp.CourseInfoResp;
import com.ym.mec.education.resp.CourseScheduleResp;
import com.ym.mec.education.service.*;
import com.ym.mec.education.utils.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程计划表 服务实现类
 * </p>
 *
 * @author lemeng
 * @since 2019-09-25
 */
@Service("ICourseScheduleService")
public class CourseScheduleServiceImpl extends ServiceImpl<CourseScheduleMapper, CourseSchedule> implements ICourseScheduleService {

    @Autowired
    private IClassGroupService groupService;
    @Autowired
    private IClassGroupTeacherMapperService classGroupTeacherMapperService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ISubjectService subjectService;
    @Autowired
    private IStudentAttendanceService studentAttendanceService;
    @Autowired
    private IMusicGroupQuitService musicGroupQuitService;

    @Autowired
    private IMusicGroupService musicGroupService;

    @Autowired
    private ISchoolService schoolService;

    @Override
    public PageResponse getPage(ClassGroupReq classGroupReq) {
        if (Objects.isNull(classGroupReq.getGroupId())) {
            return PageResponse.errorParam();
        }
        Page<CourseSchedule> courseSchedulePage = new Page<CourseSchedule>(classGroupReq.getPageNo(), classGroupReq.getPageSize());
        QueryWrapper<CourseSchedule> courseScheduleQueryWrapper = new QueryWrapper<>();
        courseScheduleQueryWrapper.lambda().eq(true, CourseSchedule::getClassGroupId, classGroupReq.getGroupId());
        IPage<CourseSchedule> courseScheduleIPage = page(courseSchedulePage, courseScheduleQueryWrapper);
        Page<CourseScheduleResp> pageResult = new Page<>();
        List<CourseScheduleResp> courseScheduleRespList = Lists.newArrayList();
        BeanUtils.copyProperties(courseScheduleIPage, pageResult);
        ClassGroup classGroup = groupService.getById(classGroupReq.getGroupId());
        courseScheduleIPage.getRecords().forEach(item -> {
            CourseScheduleResp courseScheduleResp = new CourseScheduleResp();
            courseScheduleResp.setId(item.getId());
            if (Objects.nonNull(item.getClassDate())) {
                courseScheduleResp.setClassDate(DateUtil.date2String(item.getClassDate())
                        + " " + DateUtil.date2Week(item.getClassDate()));
            }
            if (Objects.nonNull(item.getStartClassTime()) && Objects.nonNull(item.getEndClassTime())) {
                courseScheduleResp.setClassTime(DateUtil.time2String(item.getStartClassTime()) + "-" +
                        DateUtil.time2String(item.getEndClassTime()));
            }
            if (StringUtils.isNotBlank(classGroup.getSubjectIdList())) {
                List<String> subjectNameList = subjectService.getSubjectList(classGroup.getSubjectIdList()).stream().map(Subject::getName).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(subjectNameList)) {
                    courseScheduleResp.setSubjectName(subjectNameList);
                }
            }
            QueryWrapper<ClassGroupTeacherMapper> classGroupTeacherMapperQueryWrapper = new QueryWrapper<>();
            classGroupTeacherMapperQueryWrapper.lambda().eq(ClassGroupTeacherMapper::getClassGroupId, item.getClassGroupId())
                    .eq(ClassGroupTeacherMapper::getTeacherRole, TeachTypeEnum.BISHOP.getCode())
                    .eq(ClassGroupTeacherMapper::getUserId, item.getTeacherId());
            ClassGroupTeacherMapper classGroupTeacherMapper = classGroupTeacherMapperService.getOne(classGroupTeacherMapperQueryWrapper);
            if (Objects.nonNull(classGroupTeacherMapper)) {
                SysUser sysUser = sysUserService.getById(classGroupTeacherMapper.getUserId());
                if (Objects.nonNull(sysUser)) {
                    courseScheduleResp.setTeacher(sysUser.getRealName());
                }
            }
            courseScheduleRespList.add(courseScheduleResp);
        });
        pageResult.setRecords(courseScheduleRespList);
        return PageResponse.success(pageResult);
    }

    @Override
    public BaseResponse<CourseScheduleResp> getInfo(CourseScheduleReq courseScheduleReq) {
        if (Objects.isNull(courseScheduleReq.getCourseScheduleId())) {
            return BaseResponse.errorParam();
        }
        CourseScheduleResp courseScheduleResp = new CourseScheduleResp();
        CourseSchedule courseSchedule = getById(courseScheduleReq.getCourseScheduleId());
        if (Objects.nonNull(courseSchedule)) {
            if (Objects.nonNull(courseSchedule.getClassDate())) {
                courseScheduleResp.setClassDate(DateUtil.date2String(courseSchedule.getClassDate()) + "(" +
                        DateUtil.date2Week(courseSchedule.getClassDate()) + ")");
            }
            if (Objects.nonNull(courseSchedule.getStartClassTime()) &&
                    Objects.nonNull(courseSchedule.getEndClassTime())) {
                courseScheduleResp.setClassTime(DateUtil.time2String(courseSchedule.getStartClassTime()) + "-" +
                        DateUtil.time2String(courseSchedule.getEndClassTime()));
            }
            if (Objects.nonNull(courseSchedule.getTeacherId())) {
                Optional.of(sysUserService.getById(courseSchedule.getTeacherId())).
                        ifPresent(sysUser -> courseScheduleResp.setTeacher(sysUser.getRealName()));
            }
            QueryWrapper<StudentAttendance> studentAttendanceQueryWrapper = new QueryWrapper<>();
            studentAttendanceQueryWrapper.lambda().eq(true, StudentAttendance::getCourseScheduleId, courseSchedule.getId())
                    .orderByDesc(true, StudentAttendance::getCreateTime);
            List<StudentAttendance> studentAttendanceList = studentAttendanceService.list(studentAttendanceQueryWrapper);
            if (!CollectionUtils.isEmpty(studentAttendanceList)) {
                studentAttendanceList.stream().findFirst().ifPresent(studentAttendance ->
                        courseScheduleResp.setLastCommitDate(DateUtil.date2String(studentAttendance.getCreateTime(), DateUtil.DATE_FORMAT_HOUR)));
            }
            QueryWrapper<StudentAttendance> leaveWrapper = new QueryWrapper<>();
            QueryWrapper<StudentAttendance> normalWrapper = new QueryWrapper<>();
            //总人数
            Integer totalCount = count();
            //请假
            leaveWrapper.lambda().eq(true, StudentAttendance::getCourseScheduleId, courseSchedule.getId())
                    .eq(true, StudentAttendance::getStatus, StudentAttendanceStatusEnum.LEAVE.getCode());
            Integer leaveCount = studentAttendanceService.count(leaveWrapper);
            //正常
            normalWrapper.lambda().eq(true, StudentAttendance::getCourseScheduleId, courseSchedule.getId())
                    .eq(true, StudentAttendance::getStatus, StudentAttendanceStatusEnum.NORMAL.getCode());
            Integer normalCount = studentAttendanceService.count(normalWrapper);
            courseScheduleResp.setLeaveNum(leaveCount);
            if (totalCount != 0) {
                courseScheduleResp.setAttendanceRate(normalCount + "/" + totalCount);
            }
            //当前时间
            courseScheduleResp.setClassDate(DateUtil.date2String(new Date()) + " " + DateUtil.date2Week(new Date()));
        }
        return BaseResponse.success(courseScheduleResp);
    }

    @Override
    public BaseResponse getStatisticsInfo(ClassGroupReq classGroupReq) {
        if (Objects.isNull(classGroupReq.getGroupId())) {
            return BaseResponse.errorParam();
        }
        ClassGroup classGroup = groupService.getById(classGroupReq.getGroupId());
        CourseScheduleResp courseScheduleResp = new CourseScheduleResp();
        if (Objects.nonNull(classGroup)) {
            if (Objects.nonNull(classGroup.getCurrentClassTimes()) && Objects.nonNull(classGroup.getTotalClassTimes())) {
                courseScheduleResp.setAlreadyInClass(classGroup.getCurrentClassTimes() + "/" + classGroup.getTotalClassTimes());
            }
            if (Objects.nonNull(classGroup.getStudentNum())) {
                courseScheduleResp.setStudentNum(classGroup.getStudentNum());
            }
            //退团人数
            QueryWrapper<MusicGroupQuit> musicGroupQuitQueryWrapper = new QueryWrapper<>();
            musicGroupQuitQueryWrapper.lambda().eq(true, MusicGroupQuit::getMusicGroupId, classGroup.getMusicGroupId());
            int leagueNum = musicGroupQuitService.count(musicGroupQuitQueryWrapper);
            courseScheduleResp.setLeagueNum(leagueNum);
        }
        return BaseResponse.success(courseScheduleResp);
    }

    @Override
    public BaseResponse courseInfo(CourseScheduleReq courseScheduleReq) {


        if(courseScheduleReq.getCourseScheduleId() == null){
            return BaseResponse.failParams();
        }
        CourseInfoResp resp = new CourseInfoResp();
        CourseSchedule courseSchedule = this.baseMapper.selectById(courseScheduleReq.getCourseScheduleId());
        if(courseSchedule != null){
            resp.setCourseName(courseSchedule.getName());
            resp.setClassDate(DateUtil.date2String(courseSchedule.getClassDate(),DateUtil.DATE_FORMAT));
            resp.setCourseScheduleId(courseSchedule.getId());
            resp.setStartClassTime(DateUtil.date2String(courseSchedule.getStartClassTime(),DateUtil.TIME_FORMAT));
            resp.setEndClassTime(DateUtil.date2String(courseSchedule.getEndClassTime(),DateUtil.TIME_FORMAT));
            resp.setWeek(DateUtil.date2Week(courseSchedule.getClassDate()));

            SysUser sysUser = sysUserService.getById(courseSchedule.getTeacherId());
            if(sysUser != null){
                resp.setRealName(sysUser.getRealName());
            }
            ClassGroup classGroup = groupService.getById(courseSchedule.getClassGroupId());
            if(classGroup != null){
                MusicGroup musicGroup = musicGroupService.getById(courseSchedule.getClassGroupId());
                if(musicGroup != null && musicGroup.getSchoolId()!= null){
                    School school = schoolService.getById(musicGroup.getSchoolId());
                    if(school != null){
                        resp.setSchoolName(school.getName());
                    }
                }
            }


        }
        return BaseResponse.success(resp);
    }
}
