package com.ym.mec.student.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.MusicGroupSubjectGoodsAndInfoDto;
import com.ym.mec.biz.dal.dto.RegisterPayDto;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectGoodsGroup;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectPlan;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.xml.soap.Name;
import java.math.BigDecimal;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@RequestMapping("musicGroup")
@Api(tags = "乐团服务")
@RestController
public class MusicGroupController extends BaseController {

    @Resource
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private MusicGroupSubjectGoodsGroupService musicGroupSubjectGoodsGroupService;
    @Autowired
    private GoodsService goodsService;

    @ApiOperation("获取学生所在乐团列表")
    @GetMapping(value = "/queryUserMusicGroups")
    public Object queryUserMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("获取用户信息失败");
        }
        return succeed(musicGroupService.queryUserMusicGroups(sysUser.getId()));
    }

    @ApiOperation("获取学生个人中心乐团信息")
    @GetMapping(value = "/queryPersonalMusicGroups")
    public Object queryPersonalMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("获取用户信息失败");
        }
        return succeed(musicGroupService.queryPersonalMusicGroups(sysUser.getId()));
    }

    @ApiOperation(value = "根据学员编号，获取有关联的老师列表")
    @GetMapping("/findTeachersByStuId")
    public Object findTeachersByStuId() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("获取用户信息失败");
        }
        return succeed(musicGroupService.findTeachersByStuId(sysUser.getId()));
    }


    @ApiOperation(value = "获取乐团声部费用信息及乐器和辅件")
    @GetMapping("/getSubjectGoodsAndInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "subjectId", value = "声部编号", required = true, dataType = "Integer")})
    public HttpResponseResult getSubjectGoodsAndInfo(Integer musicGroupId, Integer subjectId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        MusicGroupSubjectGoodsAndInfoDto subjectGoodsAndInfo = musicGroupSubjectPlanService.getSubjectGoodsAndInfo(musicGroupId, subjectId);
        return succeed(subjectGoodsAndInfo);
    }

    @ApiOperation(value = "乐团报名支付")
    @GetMapping("/pay")
    @ApiImplicitParams({@ApiImplicitParam(name = "registerPayDto", value = "支付信息", required = true, dataType = "Integer")})
    public HttpResponseResult pay(@RequestBody RegisterPayDto registerPayDto) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        StudentRegistration studentRegistration = studentRegistrationService.get(registerPayDto.getRegisterId().longValue());
        if (!studentRegistration.getUserId().equals(sysUser.getId())) {
            return failed("报名信息有误，请核查");
        }

        BigDecimal amount = registerPayDto.getAmount();

        BigDecimal orderAmount = new BigDecimal("0");

        //获取课程价格
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId());
        orderAmount.add(musicOneSubjectClassPlan.getFee());


        //乐器及打包辅件
        if (registerPayDto.getGoodsGroupIds() != null) {
            List<MusicGroupSubjectGoodsGroup> goodsGroups = musicGroupSubjectGoodsGroupService.findGoodsGroupByIds(registerPayDto.getGoodsGroupIds());
            goodsGroups.forEach(goodsGroup -> {
                //辅件价格
                if (goodsGroup.getType().equals(GoodsType.ACCESSORIES)) {
                    orderAmount.add(goodsGroup.getPrice());
                    return;
                }
                if (musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.LEASE)) {//租赁
                    orderAmount.add(musicOneSubjectClassPlan.getDepositFee());
                } else if (musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.GROUP)) {//团购
                    orderAmount.add(goodsGroup.getPrice()).subtract(goodsGroup.getRemissionCourseFee());
                }
            });
        }

        //单独辅件
        if (registerPayDto.getGoodsIds() != null) {
            List<Goods> goodsList = goodsService.findGoodsByIds(registerPayDto.getGoodsIds());
            goodsList.forEach(goods -> {
                orderAmount.add(goods.getGroupPurchasePrice());
            });
        }
        if (amount.compareTo(orderAmount) != 0) {
            return failed("商品价格不符");
        }

        Map<String, Object> rqMap = new LinkedHashMap<String, Object>();
        rqMap.put("merNo", "07654478");
        rqMap.put("version", "v1");
        rqMap.put("notifyUrl", "https://notify.cpm");
        rqMap.put("timestamp", new Date().getTime());
        rqMap.put("apiContent", "ap");
        rqMap.put("signType", "signType");
        rqMap.put("sign", "sign");
        rqMap.put("payUrl", "http://pay.url");
        return succeed(rqMap);
    }
}
