package com.ym.mec.student.controller;

import com.ym.mec.biz.dal.dto.ClassDateAdjustDto;
import com.ym.mec.biz.dal.entity.CourseScheduleComplaints;
import com.ym.mec.biz.dal.page.StudentCourseScheduleRecordQueryInfo;
import com.ym.mec.biz.service.CourseScheduleService;
import com.ym.mec.biz.service.StudentAttendanceService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * @Author Joburgess
 * @Date 2019/9/24
 */
@RequestMapping("studentCourseSchedule")
@Api(tags = "课程计划服务")
@RestController
public class StudentCourseScheduleController extends BaseController {

    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private StudentAttendanceService studentAttendanceService;

    @ApiOperation(value = "根据月份获取乐团在该月有课的日期")
    @GetMapping("/getCourseScheduleDateByMonth")
    public Object getCourseScheduleDateByMonth(@ApiParam(value = "月份", required = true) @RequestParam Date month,
                                               @ApiParam(value = "未上课", required = true) Integer isAttend) {
        return succeed(scheduleService.getCourseScheduleDateByStudent(month,isAttend));
    }

    @ApiOperation(value = "根据日期获取当日排课")
    @GetMapping("/getCourseSchedulesWithDate")
    public Object getCourseSchedulesWithDate(@ApiParam(value = "日期", required = true) Date date){
        return succeed(scheduleService.getStudentCourseSchedulesWithDate(date));
    }

    @ApiOperation(value = "课时调整")
    @PostMapping(value = "/classStartDateAdjust",consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Object classStartDateAdjust(ClassDateAdjustDto classDateAdjustDto){
        scheduleService.classStartDateAdjust(classDateAdjustDto);
        return succeed();
    }

    @ApiOperation(value = "课时交换")
    @GetMapping(value = "/courseSwap",consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Object courseSwap(Long courseScheduleId1,Long courseScheduleId2){
        scheduleService.courseSwap(courseScheduleId1,courseScheduleId2);
        return succeed();
    }

    @ApiOperation(value = "上课记录")
    @GetMapping(value = "/findStudentCourseScheduleRecords")
    public Object findStudentCourseScheduleRecords(StudentCourseScheduleRecordQueryInfo queryInfo){
        return succeed(scheduleService.findStudentCourseScheduleRecords(queryInfo));
    }

    @ApiModelProperty("课程投诉")
    @PostMapping("/courseScheduleCommplaint")
    public Object courseScheduleCommplaint(CourseScheduleComplaints courseScheduleComplaints){
        scheduleService.courseScheduleCommplaint(courseScheduleComplaints);
        return succeed();
    }

}
